class Retry:
    """
    class Retry : This will retry the function by 1 time.
    """

    def __init__(self, except_function=None, retry_value=1):
        """
        Retry class : Default retry value is 1

        :param except_function:
        :param retry_value:
        """
        self.except_function = except_function
        self.retry_value = retry_value

    def __call__(self, function):
        """
        Calling function with arguments.
        Default retry is 1.
        Maximum 1 failure is ignored.

        :param function: Foo
        :param args: Foo args
        :param kwargs: Foo kwargs

        :return:
        """

        def inner_func(*args, **kwargs):
            """
            Inner function which executes the actual function and Except function.

            :param args:
            :param kwargs:
            :return:
            """
            if self.retry_value == 0:
                return
            for i in range(0, self.retry_value + 1):
                try:
                    return function(*args, **kwargs)
                except Exception as e:
                    if i == self.retry_value:
                        raise e
                    if self.except_function:
                        self.except_function(*args, **kwargs)
                    print("Try #{} failed with Exception: {}".format(i, e))
                    continue

        return inner_func


@Retry(retry_value=0)
def x_fun():
    print("Function is disabled")



class BaseClass:

    def __init__(self, function):
        self.function = function
        self.obj = None

    def __instancecheck__(self, dec_class):
        return self.obj


class CamelCase(BaseClass):

    def __call__(self, *args, **kwargs):
        try:
            self.obj = self.function(*args, **kwargs)
            func_dict = self.obj.__dict__
            class_var_value = {var: func_dict[var] for var in func_dict}
            for _var in class_var_value:
                r_var = self.camel_type(_var)
                self.obj.__setattr__(r_var, class_var_value[_var])
                if r_var != _var:
                    self.obj.__delattr__(_var)

            return self.obj

        except Exception as e:
            print("Error while converting to camel case : {}".format(e))
            raise e

    @staticmethod
    def camel_type(snake_str):
        try:
            snake_str_list = snake_str.split("_")
            if len(snake_str_list) > 1:
                return str(snake_str_list[0]) + str("".join(x.title() for x in snake_str_list[1:]))
            else:
                return snake_str_list[0]
        except Exception as e:
            print("Error While converting to camel case : {}".format(e))
            raise e


class PascalCase(BaseClass):

    def __call__(self, *args, **kwargs):
        try:
            self.obj = self.function(*args, **kwargs)
            func_dict = self.obj.__dict__
            class_var_value = {var: func_dict[var] for var in func_dict}
            for _var in class_var_value:
                r_var = self.pascal_type(_var)
                self.obj.__setattr__(r_var, class_var_value[_var])
                if r_var != _var:
                    self.obj.__delattr__(_var)

            return self.obj

        except Exception as e:
            print("Error while converting to pascal case : {}".format(e))
            raise e

    @staticmethod
    def pascal_type(snake_str):
        try:
            snake_str_list = snake_str.split("_")
            return "".join(x.title() for x in snake_str_list)
        except Exception as e:
            print("Error While converting to pascal case : {}".format(e))
            raise e
